function NM4M_CompFeat(DIR,p)

%This function gets basis representations from wav files by using NMF and writes them to a separate
% file for each audio file. It
%is assumed that wav files belonging to a particular class are contained in
%separate folders in DIR. The sampling frequency of the files has to be
%16000Hz and the files have to be mono.
%NM4M_CompFeat works with differing numbers of components for each frame
%when p.percentage is set to a value bigger than fourty. If it is smaller
%the value is taken for the desired number of components.
% The spectral-temporal representation of the signal can be either a
% Sonogram or an amplitude spectrum on a logarithmic frequency axis.
% Inputs:
% DIR       folder with the data in wav format
% p         parameter structure (for docu see NM4M_ParamInit.m)
% The structure p enlarges the structure defined by Elias Pampalk by the fields:
% p.bpo : the desired number of Bands per octave for the sonogram,
% default=8
% p.winlength : the desired window length for the fft in seconds,
% default=0.04
% p.overlap : the overlap of the fft windows, default=0.5
% p.block : length of the timbre window in seconds, default=0.5
% p.decomp_meth : choice of algorithm for the matrix decomposition
%       value 1          : EM algorithm for NMF as described by Smaragdis
%       value 2          : Use fluctuation patterns by Elias Pampalk, just for debugging!
%       value 3 (default): NMF with divergence criterium as implemented by
%       Hoyer
%       value 4          : for debugging only
% p.percentage : desired percentage of singular value sum to be contained,
% in approximation. This has influence on the number of components.
% default=3, set to value smaller than 40 for a constant component number
% p.fs : sample frequency of the audio files, default=16000
% p.dB_max : SPL/dB the audio files are normalized to, default=96
% p.do_visu : visualization on/off, default=0
%p.feature : set to one for spectral components as described in my ICASSP
%paper, set to three for storing the temporal weights for usage in rhythm
%descriptors
% for full docu of the parameter structure see file NM4M_ParamInit.m
%
% Andre Holzapfel
% Master Thesis Implementation
% Signal Processing Laboratory
% University of Crete, June 2006
% Supervisor: Jannis Stylianou

%   Copyright (c) 2008 University of Crete.
% 
%   This file is part of Non-negative Matrix Factorization for Music (NM4M)
% 
%   NM4M is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License version 2 as
%   published by the Free Software Foundation.
% 
%   NM4M is distributed in the hope that it will be useful, but
%   WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%   General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with NM4M; if not, write to the Free Software
%   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
%   02110-1301 USA
% 
%   See the file "COPYING" for the text of the license.

error(nargchk(1,3,nargin));%nargchk(minargs, maxargs, numargs)

p = NM4M_ParamInit(p);%set parameters to standard values if not set

NumBands = 7*p.bpo;%take seven octaves

store_sone = 0; %set this to one if you want to save all the sonograms!
if((p.percentage) > 100 && p.decomp_meth ~=1)
    error('Learning of overcomplete set only possible for NMF Extraction!');
end

% Get directory structures for each sound class
D = dir(DIR);k=0;
for i=1:length(D)
    if(D(i).isdir & ~(D(i).name(1) == '.'))
        k=k+1;
        soundDir(k)=D(i);
    end
end
if strcmp(p.spec,'logF')
    baseparmstr = ['_' num2str(p.percentage) '_' num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) ...
        '_' num2str(p.block) '_' num2str(p.decomp_meth) p.spec '.mat'];
elseif strcmp(p.spec,'sone')
    baseparmstr = ['_' num2str(p.percentage) '_' num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) ...
        '_' num2str(p.block) '_' num2str(p.decomp_meth) p.spec '_' ...
        p.outerear '_' num2str(p.do_spread) '.mat'];
end

if(k>0)
    fixed = 0;
    for i=1:k
        ClassName{i} = fullfile( DIR, soundDir(i).name) ;
        fprintf('\nConstructing feature vectors for: %s\n', ClassName{i});
        TL=dir([ClassName{i} '/*.wav']);
        for j=1:length(TL)
            index = 1;
            AllList{i}.file{j}=fullfile(DIR,soundDir(i).name,TL(j).name);
            if(~exist([AllList{i}.file{j}(1:length(AllList{i}.file{j})-4) baseparmstr]))
                fprintf('\n File: %s \n',AllList{i}.file{j});
                [wav_all,p.fs] = wavread(AllList{i}.file{j});
                if (p.fs ~= 16000)
                    error('Wrong sampling frequency of the input file, 16kHz is assumed!');
                end
                %wav_all = wav_all( (p.fs) : (length(wav_all)-p.fs));%leave first and last second to avoid lead in/out effects
                %using normalization from the HUTear toolbox
                wav_all = remmean(wav_all);
                if ~isempty(find(p.decomp_meth== [2 3]))%ICA has problems with this...
                    wav_all = Pascalize(wav_all,p.dB_max);
                end
                %improved way to delete silence and fade in effect at beginning
                %and end of file
                while(mean(my_dB(wav_all(1:(p.block*p.fs)),1)) < (p.dB_max-40))
                    wav_all = wav_all((p.block*p.fs):length(wav_all));
                end
                wav_all = wav_all((p.block*p.block_hop*p.fs):length(wav_all));
                while(mean ( my_dB( wav_all( (length(wav_all)-(p.block*p.fs)+1) : length(wav_all)),1)) < (p.dB_max-40))
                    wav_all = wav_all(1:(length(wav_all)-(p.block*p.fs)+1));
                end
                wav_all = wav_all(1:(length(wav_all)-(p.block*p.fs)+1));
                duration = length(wav_all);%must be mono!
                snack = 30*p.fs;%this is to process big files piecewise. Especially poor Windows users experience less Out of Memories then :o)
                numsnacks = ceil(duration/snack);
                begin = 1;
                ending = snack;
                for h = 1:numsnacks
                    begin = (h-1)*snack+1;
                    if ((h*snack) > duration)
                        ending = duration;
                    else
                        ending = h * snack;
                    end
                    wav = wav_all(begin:ending);

                    switch p.spec
                        case 'sone'
                            sone = NM4M_PsychModel(wav,p);
                        case 'logF'
                            sone = AudioSpectrumEnvelope(wav',p.fs,p.bpo,0,p.winlength,p.overlap);
                            sone = sone([1 3:size(sone,1)],:); %compensate a bug in Casey's code
                            sone = sone(2:(size(sone,1)-1),:);%remove the lowest and the upper band
                    end
                    if (min(min(sone)) <=0)%set zeros to positive values for NMF to work
                        indices = find(sone <= 0);
                        fprintf('class %i song %i : %s, %i zeros corrected \n',i,j,AllList{i}.file{j},length(indices));
                        sone(indices) = 1e-12;
                    end
                    sone = flipud(sone);%for reasons of a logarithmic plot, avoiding log(zero)
                    if store_sone
                        save([AllList{i}.file{j}(1:length(AllList{i}.file{j})-4)  '_'...
                            num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) '_' p.spec '.mat'],'sone');
                    end
                    BlockSamp = p.block*p.fs;%How many samples to be contained in a p.block
                    NumFrames = floor(length(wav)/(BlockSamp*p.block_hop));%For how many frames do we have to compute a deconvolution,
                    clear wav;
                    %BlockHop = floor(size(sone,2)/NumFrames);%length of the block in the sonogram
                    BlockSize = floor(1/(p.winlength*p.overlap)*p.block);
                    BlockHop = floor(BlockSize * p.block_hop);
                    for l = 1:NumFrames
                        if p.percentage >= 100
                            Ncomps = floor(size(sone,1)*(p.percentage/100));%learn overcomplete set
                        elseif p.percentage < 40
                            Ncomps = p.percentage;%set to a constant value
                        else %get number of components by an svd estimation
                            if ((l==NumFrames) && (h==numsnacks) )
                                Ncomps = GetNumComps(sone(:,((l-1)*BlockHop+1):(size(sone,2))),p.percentage/100);
                            else
                                Ncomps = GetNumComps(sone(:,((l-1)*BlockHop+1):(((l-1)*BlockHop)+BlockSize)),p.percentage/100);
                            end
                        end
                        if ( (l*BlockHop) > size(sone,2) )
                            indices = ((l-1)*BlockHop+1):(size(sone,2));
                        else
                            indices = ((l-1)*BlockHop+1):(l*BlockHop);
                        end
                        if (p.decomp_meth==1)%calc NMF with EM algorithm by Smaragdis
                            [U,A,V] = svd(sone(:,indices)',0);%X = U*A*V'
                            V1=V(:,1:Ncomps);
                            Y1=sone(:,indices)'*V1;
                            V2=eye(Ncomps);
                            Y2=Y1; % Initialize output matrices
                            delta=diag(A(:,1:Ncomps));
                            [XX,mix,demix] = fastica(Y1', 'approach', 'symm', ...
                                'verbose', 'off', 'displayMode', 'off');%[Components,pseudoinverse of W,seperating matrix
                            %[XX,mix,demix] = fastica(sone(:,indices), 'approach', 'symm', 'verbose', 'off','displayMode', 'off', 'lastEig', Ncomps);
                            if( isempty( find( mix>eps ) ) | isnan(mix) | isinf(mix))
                                fprintf('warning: fixing mixing matrix...');
                                mix=pinv(demix);
                                fixed = fixed+1;
                            end
                            % Use estimated unmixing matrix as stored basis
                            W(index).data = mix;
                            H(index).data = XX;
                        elseif (p.decomp_meth == 2)%just take random vecs, for comparison!
                            which = randperm(length(indices));
                            W(index).data = sone(:,indices(which(1:Ncomps)));
                        elseif (p.decomp_meth == 3)%use NMF with divergence criterium from Hoyer toolbox, for comparison
                            if ( ((l-1)*BlockHop+BlockSize) > size(sone,2) )
                                if p.feature == 3
                                    break;% FOR HORIZONTAL FEATURES STOP HERE TO HAVE EQUAL SIZES OF COEFF MATRICES 
                                end
                                [W(index).data,H(index).data] = NM4M_Decomp( sone(:,((l-1)*BlockHop+1):(size(sone,2))),Ncomps);
                            else
                                [W(index).data,H(index).data] = NM4M_Decomp( sone(:,((l-1)*BlockHop+1):(((l-1)*BlockHop)+BlockSize)),Ncomps);
                            end
                        end
                        if p.dB_flag
                            if p.spec == 'sone'
                                error('dB-flag set when using sonogram!');
                            else
                                W(index).data = my_dB(W(index).data.^.5,p.dB_min); % Convert power to decibel scale, everything beneath 40dB SPL is clipped
                            end
                        end
                        if (p.baseCC == 1)
                            W(index).data = NM4M_BaseCC(W(index).data,p);%convert to cepstral representation
                        end
                        index = index + 1;
                    end
                end
                if p.feature == 1
                    if strcmp(p.spec,'logF')
                        save([AllList{i}.file{j}(1:length(AllList{i}.file{j})-4) '_' num2str(p.percentage) '_'...
                            num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) '_' num2str(p.block) '_' num2str(p.decomp_meth) p.spec '.mat'],'W');
                    elseif strcmp(p.spec,'sone')
                        save([AllList{i}.file{j}(1:length(AllList{i}.file{j})-4) '_' num2str(p.percentage) '_'...
                            num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) '_' num2str(p.block) '_' num2str(p.decomp_meth) p.spec '_' ...
                            p.outerear '_' num2str(p.do_spread) '.mat'],'W');
                    end
                elseif p.feature == 3
                    save([AllList{i}.file{j}(1:length(AllList{i}.file{j})-4) '_' num2str(p.percentage) '_'...
                        num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) '_' num2str(p.block) '_' num2str(p.decomp_meth) p.spec '.mat'],'H');
                end
            else
                fprintf('\nFound feature vectors for: %s', AllList{i}.file{j});
            end
            %save([AllList{i}.file{j}(1:length(AllList{i}.file{j})-4) '_' num2str(p.percentage) '_'...
            %num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) '_' num2str(p.block) '_' num2str(p.decomp_meth) p.spec '_Coeff.mat'],'H');
            clear H; clear W; clear sone;clear wav_all;
        end
    end
else
    fprintf('\nBummer! No sound directories found\n');
end
