function [W,H] = NM4M_Decomp( V, rdim)
% Non negative matrix factorization using a divergence criteria function.
% The Algorithm from the NMFTools by Patrik O. Hoyer has been used and has
% been adapted to the needs of our framework.
%Inputs:
% rdim      number of components to be calculated
% V         input spectrogram, [NumberOfFreqBins x TimeSlices]
% Outputs:
% W         spectral base
% H         temporal base
%
% Andre Holzapfel
% Master Thesis Implementation
% Signal Processing Laboratory
% University of Crete, June 2006
% Supervisor: Jannis Stylianou

%   Copyright (c) 2008 University of Crete.
% 
%   This file is part of Non-negative Matrix Factorization for Music (NM4M)
% 
%   NM4M is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License version 2 as
%   published by the Free Software Foundation.
% 
%   NM4M is distributed in the hope that it will be useful, but
%   WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%   General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with NM4M; if not, write to the Free Software
%   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
%   02110-1301 USA
% 
%   See the file "COPYING" for the text of the license.

miniter = 10;
minGrad = 0.01;
% Check that we have non-negative data
if min(V(:))<0, error('Negative values in data!'); end
% Dimensions
vdim = size(V,1);
samples = size(V,2);
% Create initial matrices
W = abs(randn(vdim,rdim));
H = abs(randn(rdim,samples));
% Calculate initial objective
objhistory = sum(sum((V.*log(V./(W*H))) - V + W*H));
% Start iteration
iter = 0;
while 1,
    % Update iteration count
    iter = iter+1;    
    % Save old values
    Wold = W;
    Hold = H;
    % Compute new W and H (Lee and Seung; NIPS*2000)
    W = W.*((V./(W*H + 1e-9))*H')./(ones(vdim,1)*sum(H'));
    H = H.*(W'*(V./(W*H + 1e-9)))./(sum(W)'*ones(1,samples));
    % Renormalize so rows of H have constant energy
    norms = sqrt(sum(H'.^2));
    H = H./(norms'*ones(1,samples));
    W = W.*(ones(vdim,1)*norms);
    % Calculate objective
    newobj = sum(sum((V.*log(V./(W*H))) - V + W*H));
    objhistory = [objhistory newobj];
    if (iter > miniter)
        if (abs(objhistory(iter)-objhistory(iter+1))/objhistory(iter+1) < minGrad)    
            break;
        end
		if (newobj == Inf)
            fprintf('Approximation stopped due to divison by zero. Most probably caused by a signal being zero.');%if this appears 
            %take care that you cut away the silence at the beginning and
            %at the end of a piece!
            break;
        end
    end
end