function [confusion_ismir,true] = NM4M_TestBase(DIR,p,ALT_NAME)
%Classifies audio data based on their spectral bases computed by a
%decomposition in NM4M_CompFeat. A GMM model structure is read which has to
%be trained by NM4M_TrainBase before.
% Inputs:
% DIR       folder with the spectral base data
% p         parameter structure (for docu see NM4M_ParamInit.m)
% ALT_NAME  specifies the data list names and the name of the classifier to
%           be read
% Outputs:
% confusion _ismir  contains the number of votes for every class as used in
%                   the ISMIR contest of 2004
% true              percentages of the correct classifications for each
%                   class
%
%
% Andre Holzapfel
% Master Thesis Implementation
% Signal Processing Laboratory
% University of Crete, June 2006
% Supervisor: Jannis Stylianou

%   Copyright (c) 2008 University of Crete.
% 
%   This file is part of Non-negative Matrix Factorization for Music (NM4M)
% 
%   NM4M is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License version 2 as
%   published by the Free Software Foundation.
% 
%   NM4M is distributed in the hope that it will be useful, but
%   WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%   General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with NM4M; if not, write to the Free Software
%   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
%   02110-1301 USA
% 
%   See the file "COPYING" for the text of the license.


p = NM4M_ParamInit(p);%set parameters to standard values if not set
baseparmstr = ['_' num2str(p.percentage(1)) '_' num2str(p.bpo) '_' num2str(p.winlength) '_' num2str(p.overlap) ...
    '_' num2str(p.block) '_' num2str(p.decomp_meth) '.mat'];%parameter string for recognizing the correct features
TEST_LIST_NAME = [ALT_NAME '_TestList.txt'];
D = dir(DIR);k=0;
for i=1:length(D)
    if(D(i).isdir & ~(D(i).name(1) == '.'))
        k=k+1;
        soundDir(k)=D(i);
    end
end
open(fullfile(DIR,['bayesS_' ALT_NAME '_' num2str(p.num_clusters) '_' num2str(p.crossrun) '.mat']));
bayesS = ans.bayesS;%get the trained classifier
if (length(p.percentage)==1)%fixed compression case
    perc_length = 1;
elseif (length(p.percentage)==k)%varying compression
    perc = unique(p.percentage);
    perc_length = length(perc);
else
    error(sprintf(['The percentage field has to be either a single number for equal compression of all classes \n' ...
        'or a vector of the same length as the number of classes for class specific compression']));
end
if(k>0)
    skipped = 0;
    skippedind = [];
    confusion = zeros(k,k);
    confusion_ismir = zeros(k,k);
    for i=1:k
        ClassName{i} = fullfile( DIR, soundDir(i).name) ;
        fprintf('\nClassifying Songs in: %s\n', ClassName{i});
        sfxfilename = fullfile( ClassName{i}, TEST_LIST_NAME );
        fileList = readsfxfiles(sfxfilename);
        if isempty(fileList)
            fprintf('Skipping folder %s because it contains no files!\n',ClassName{i});
            skipped = skipped + 1;
            continue;
        end
        if p.print_acc%if set to one a file for each class is written with details of the classification for all songs
            fid = fopen([ClassName{i} '_' ALT_NAME '_' num2str(p.num_clusters) '_' num2str(p.feature) '_res.txt'],'a');
        end
        for j=1:length(fileList)
            histogram = zeros(k,1);
            for m = 1:perc_length
				if (perc_length > 1)
                	fileList2 = regexprep(fileList,['_' num2str(p.percentage(i)) '_'],['_' num2str(perc(m)) '_']);
                else
                    fileList2 = fileList;
                end
                open(fullfile(ClassName{i},fileList2{j}));
                tmp = ans.W;
                W = cat(2,tmp(1:length(tmp)).data);%read the timbre frame bases and put them together in chronological order
                if p.decomp_meth == 2
                    W = W - repmat(mean(W,2),1,size(W,2));
                    C_xx = princomp(W');
                    W = (W' * C_xx(:,1:20))';
                end
                pxomega = gmmb_pdf(W', bayesS);
                tulo = gmmb_weightprior(pxomega, bayesS);
                norm = sum(tulo,2);
                tulo = tulo ./ repmat(norm,1,k);
                tmp = sum(tulo)';
                histogram = histogram + tmp./sum(tmp);
            end
            confusion(i,:) = confusion(i,:) + histogram';
            %check the old approach, does not work when perc_length not
            % equal to one
            new_tulo = log(pxomega);
            for n=1:size(new_tulo,2)
                loglike(n) = sum(new_tulo(:,n));
            end
            [class(i).contrib(j),class(i).vote(j)] = max(loglike);%comment and uncomment this in change with line 95
            %[class(i).contrib(j),class(i).vote(j)] = max(histogram);
            if p.print_acc
                fprintf(fid,'%s; %s; %s; %f; ',fileList{j},soundDir(i).name,soundDir(class(i).vote(j)).name,loglike);%change accordingly
                fprintf(fid,'\n');
            end
        end
        confusion_ismir(i,:) = histc(class(i).vote,1:k);
        true(i) = length(find(class(i).vote==i))/length(fileList);
        fprintf('\nIn class %s %d per cent have been classified correctly\n', ClassName{i},true(i)*100);
        if p.print_acc
            fclose(fid);
        end
    end
    if skipped == 0
        perform = sum(true)/length(true);
    else
        perform = sum(true)/(length(true)-skipped);
    end
    fprintf('\n Test over all classes achieved a correctness of %d per cent \n', perform *100);
else
    fprintf('\nOoooops! There are no appropriate files.\n');
end


